MODULE WFT;

        (********************************************************)
        (*                                                      *)
        (*                    Web Family Tree                   *)
        (*                                                      *)
        (*     This is a CGI program, intended to be invoked    *)
        (*               from a web browser                     *)
        (*                                                      *)
        (*  Programmer:         P. Moylan                       *)
        (*  Started:            6 July 2001                     *)
        (*  Last edited:        28 November 2001                *)
        (*  Status:             OK                              *)
        (*                                                      *)
        (********************************************************)

IMPORT IOChan, TextIO, Strings;

FROM SYSTEM IMPORT
    (* proc *)  ADR;

FROM ProgramArgs IMPORT
    (* proc *)  ArgChan, IsArgPresent, NextArg;

FROM STextIO IMPORT
    (* proc *)  WriteChar, WriteString, WriteLn;

FROM SysClock IMPORT
    (* type *)  DateTime,
    (* proc *)  GetClock;

FROM Person IMPORT
    (* type *)  PersonData,
    (* proc *)  GetPersonData, DiscardPersonData, DisplayPerson,
                DisplayDescendants, DisplayAncestors, DisplayEveryone;

FROM OurTypes IMPORT
    (* type *)  IDString, DataString, FilenameString;

(********************************************************************************)

CONST
    version = "0.4";
    Debugging = FALSE;
    DefaultInput = "D=Moylan;P=I027";

TYPE ViewType = (normal, ancestors, descendants, everyone);

(********************************************************************************)

PROCEDURE GetArguments (VAR (*OUT*) DatabaseName: FilenameString;
                        VAR (*OUT*) PersonID: IDString;
                        VAR (*OUT*) View: ViewType;
                        VAR (*OUT*) ShowMore: BOOLEAN);

    (* Picks up the arguments to the program. *)

    CONST Nul = CHR(0);  ArgSeparator = ';';

    VAR arg: ARRAY [0..255] OF CHAR;
        j: CARDINAL;

    (****************************************************************************)

    PROCEDURE GetString (VAR (*OUT*) str: ARRAY OF CHAR);

        (* Loads a string from arg, skipping any initial '=', stopping at end   *)
        (* of string or when ArgSeparator found.                                *)

        VAR k: CARDINAL;

        BEGIN
            IF arg[j] = '=' THEN INC(j) END(*IF*);
            k := 0;
            WHILE (k <= HIGH(str)) AND (j <= 255) AND (arg[j] <> Nul)
                      AND (arg[j] <> ArgSeparator) DO
                str[k] := arg[j];
                INC (j);
                INC (k);
            END (*WHILE*);

            (* Strip trailing spaces. *)

            WHILE (k > 0) AND (str[k-1] = ' ') DO
                DEC (k);
            END (*WHILE*);

            IF k <= HIGH(str) THEN
                str[k] := Nul;
            END (*IF*);

        END GetString;

    (****************************************************************************)

    TYPE CharSet = SET OF CHAR;

    CONST Digits = CharSet {'0'..'9'};

    VAR cid: IOChan.ChanId;  ch: CHAR;

    BEGIN
        IF Debugging THEN
            arg := DefaultInput;
        ELSE
            cid := ArgChan();
            IF IsArgPresent() THEN
                TextIO.ReadString (cid, arg);
            ELSE
                arg := "";
            END (*IF*);
        END (*IF*);

        DatabaseName := "";
        PersonID := "";
        View := normal;
        ShowMore := FALSE;

        j := 0;
        LOOP
            ch := arg[j];  INC(j);
            CASE CAP(ch) OF
                Nul:      EXIT (*LOOP*);
              | ArgSeparator:
                          (* do nothing *)
              | 'D':      GetString (DatabaseName);
              | 'P':      GetString (PersonID);
              | 'V':      IF arg[j] = '=' THEN INC(j) END(*IF*);
                          IF arg[j] = 'D' THEN
                              View := descendants;  INC(j);
                          ELSIF arg[j] = 'A' THEN
                              View := ancestors;  INC(j);
                          ELSIF arg[j] = 'E' THEN
                              View := everyone;  INC(j);
                          END(*IF*);

                          IF arg[j] = '+' THEN
                              ShowMore := TRUE;  INC(j);
                          END(*IF*);
            ELSE
                (* Ignore unknown option. *)
                INC(j);
            END (*CASE*);
        END (*LOOP*);

    END GetArguments;

(********************************************************************************)

PROCEDURE WriteCard (value: CARDINAL);

    (* Writes a left-justified cardinal to standard output. *)

    BEGIN
        IF value > 9 THEN
            WriteCard (value DIV 10);  value := value MOD 10;
        END (*IF*);
        WriteChar (CHR(ORD("0")+value));
    END WriteCard;

(********************************************************************************)

PROCEDURE WriteDate;

    (* Writes today's date to standard output. *)

    TYPE MonthNameType = ARRAY [0..12] OF ARRAY [0..2] OF CHAR;

    CONST MonthName = MonthNameType {'???', 'Jan', 'Feb', 'Mar', 'Apr', 'May',
                                     'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov',
                                     'Dec'};

    VAR now: DateTime;  k: CARDINAL;

    BEGIN
        GetClock (now);
        WriteCard (now.day);
        WriteChar (' ');
        k := now.month;
        IF k > 12 THEN k := 0 END (*IF*);
        WriteString (MonthName[k]);
        WriteChar (' ');
        WriteCard (now.year);
    END WriteDate;

(********************************************************************************)

PROCEDURE SendLine (line: ARRAY OF CHAR);

    (* Sends a string, terminated with CRLF, to standard output. *)

    BEGIN
        WriteString (line);  WriteLn;
    END SendLine;

(********************************************************************************)

PROCEDURE DisplayWebPage (found: BOOLEAN;  ID: IDString;  data: PersonData;
                                   DatabaseName: FilenameString;
                                   View: ViewType;  ShowDetails: BOOLEAN);

    (* Displays the information in the PersonData record. *)

    BEGIN
        (* Send the HTTP response. *)

        SendLine ("Content-type: text/html; charset=IBM850");
        (*
        SendLine ("Cache-Control: private");
        SendLine ("Pragma: no-cache");
        SendLine ("Expires: NOW");
        *)
        SendLine ("");

        (* Send the data as an HTML document. *)

        SendLine ("<html><head><title>");
        IF found THEN
            IF View = everyone THEN
                WriteString ("Listing everyone in the database");
            ELSE
                WriteString ("WFT data for person record ");
                WriteString (ID);
            END (*IF*);
            SendLine ("</title></head>");
            IF View = normal THEN
                DisplayPerson (data);
            ELSIF View = descendants THEN
                DisplayDescendants (data, ShowDetails);
            ELSIF View = ancestors THEN
                DisplayAncestors (data, ShowDetails);
            ELSIF View = everyone THEN
                DisplayEveryone (data, ShowDetails);
            END (*IF*);
        ELSE
            WriteString ("");
            SendLine ("Record not found</title></head>");
            SendLine ('<body text="#000000" bgcolor="#FFFFCC" link="#0000EE" vlink="#551A8B" alink="#FF0000">');
            SendLine ("<h2>Record not found</h2>");
            SendLine ("Nonexistent database file, or nonexistent individual record.");
            WriteString ("<br>Database name = '");
            WriteString (DatabaseName);
            SendLine ("'");
        END (*IF*);

        (* Finish off the HTML code. *)

        WriteString ('<p><hr><font size=-1>Generated by <A href="http://eepjm.newcastle.edu.au/os2/wft.html">WFT</a> version ');
        WriteString (version);  WriteString (" on ");
        WriteDate();
        SendLine ("</font>");
        SendLine ("</body></html>");

    END DisplayWebPage;

(********************************************************************************)

PROCEDURE DisplayInfo;

    (* Displays the information for one person. *)

    VAR PersonInfo: PersonData;
        DatabaseName: FilenameString;
        PersonID: IDString;
        View: ViewType;
        found, ShowDetails: BOOLEAN;

    BEGIN
        GetArguments (DatabaseName, PersonID, View, ShowDetails);
        found := GetPersonData (DatabaseName, PersonID, PersonInfo);
        DisplayWebPage (found, PersonID, PersonInfo, DatabaseName,
                                View, ShowDetails);
        DiscardPersonData (PersonInfo);
    END DisplayInfo;

(********************************************************************************)
(*                                 MAIN PROGRAM                                 *)
(********************************************************************************)

BEGIN
    DisplayInfo;
END WFT.

